from .model import Model

class OneClickConfig(Model):

    def _accepted_params(self):
        return [
            'configId',
            'shortNumber',
            'merchantId',
            'serviceCode',
            'businessModel',
            'preAuthServiceId',
            'preAuthServiceDescription',
            'recurring',
            'redirectUrl',
            'onlineText',
            'offlineText',
            'age',
            'isRestricted',
            'invoiceText',
            'price',
            'subscriptionPrice',
            'subscriptionInterval',
            'subscriptionStartSms',
            'timeout',
            'created',
            'lastModified',
        ]



from .model import Model

class OutMessageStrex(Model):


    def _accepted_params(self):
        return [
            'merchantId',
            'serviceCode',
            'businessModel',
            'preAuthServiceId',
            'preAuthServiceDescription',
            'age',
            'isRestricted',
            'smsConfirmation',
            'invoiceText',
            'price',
            'timeout',
            'billed',
            'resultCode',
            'resultDescription',
        ]



from .model import Model

class PublicKey(Model):

    def _accepted_params(self):
        return [
            'accountId',
            'name',
            'expiry',
            'signAlgo',
            'hashAlgo',
            'publicKeyString',
            'notUsableBefore',
            'created',
            'lastModified',
        ]



from .model import Model

class Keyword(Model):

    def _accepted_params(self):
        return [
            'keywordId',
            'shortNumberId',
            'keywordText',
            'mode',
            'forwardUrl',
            'enabled',
            'created',
            'lastModified',
            'tags',
            'customProperties',
            'aliases',
        ]



from .model import Model

class LookupResult(Model):

    def _accepted_params(self):
        return [
            'created',
            'msisdn',
            'landline',
            'firstName',
            'middleName',
            'lastName',
            'companyName',
            'companyOrgNo',
            'streetName',
            'streetNumber',
            'streetLetter',
            'zipCode',
            'city',
            'gender',
            'dateOfBirth',
            'age',
            'deceasedDate',
        ]



from .model import Model

class DeliveryReport(Model):

    def _accepted_params(self):
        return [
            'correlationId',
            'sessionId',
            'transactionId',
            'price',
            'sender',
            'recipient',
            'operatorId',
            'statusCode',
            'detailedStatusCode',
            'delivered',
            'billed',
            'smscTransactionId',
            'smscMessageParts',
        ]



from .model import Model
from .out_message_strex import OutMessageStrex

class OutMessage(Model):

    def _init_preprocess(self, args):

        # NOTE args.strex could already be a model (passed in by programmer to construsctor)
        # or it could be a dict (passed during JSON deserialization process also via constructor)
        # In this case we will convert it to a model

        if 'strex' in args:
            if type(args['strex']) != OutMessageStrex:
                strex = OutMessageStrex(**args['strex'])
                setattr(self, 'strex', strex)
                del args['strex']

        return args


    def _accepted_params(self):
        return [
            'transactionId',
            'sessionId',
            'correlationId',
            'keywordId',
            'sender',
            'recipient',
            'content',
            'strex', # OutMessageStrex class
            'allowUnicode', # TRUE to allow unicode SMS, FALSE to fail if content is unicode, NULL to replace unicode chars to '?'
            'sendTime',
            'timeToLive',
            'priority',
            'deliveryMode',
            'deliveryReportUrl',
            'statusCode',
            'smscTransactionId',
            'detailedStatusCode',
            'delivered',
            'operatorId',
            'smscMessageParts',
            'tags',
            'properties',
            'lastModified',
            'created',
        ]



from .model import Model

class OneTimePassword(Model):

    def _accepted_params(self):
        return [
            'transactionId',
            'merchantId',
            'recipient',
            'sender',
            'recurring',
            'messagePrefix',
            'messageSuffix',
            'message', # deprecated
            'timeToLive',
            'created',
            'delivered',
        ]



from .model import Model

class StrexMerchant(Model):

    def _accepted_params(self):
        return [
            'merchantId',
            'shortNumberIds', # Array of shortNumberId like 'NO-2002'
            'password',
        ]



from .model import Model

class StrexRegistrationSms(Model):

    def _accepted_params(self):
        return [
            'transactionId',
            'recipient',
            'merchantId',
            'smsText',
        ]



class DetailedStatusCodes:
	NONE = "None"
	DELIVERED = "Delivered"
	EXPIRED = "Expired"
	UNDELIVERED = "Undelivered"
	UNKNOWN_ERROR = "UnknownError"
	OTHER_ERROR = "OtherError"
	REJECTED = "Rejected"
	UNKNOWN_SUBSCRIBER = "UnknownSubscriber"
	SUBSCRIBER_UNAVAILABLE = "SubscriberUnavailable"
	SUBSCRIBER_BARRED = "SubscriberBarred"
	INSUFFICIENT_FUNDS = "InsufficientFunds"
	REGISTRATION_REQUIRED = "RegistrationRequired"
	UNKNOWN_AGE = "UnknownAge"
	DUPLICATE_TRANSACTION = "DuplicateTransaction"
	SUBSCRIBER_LIMIT_EXCEEDED = "SubscriberLimitExceeded"
	MAX_PIN_RETRY = "MaxPinRetry"
	INVALID_AMOUNT = "InvalidAmount"
	ONE_TIME_PASSWORD_EXPIRED = "OneTimePasswordExpired"
	ONE_TIME_PASSWORD_FAILED = "OneTimePasswordFailed"
	SUBSCRIBER_TOO_YOUNG = "SubscriberTooYoung"
	TIMEOUT_ERROR = "TimeoutError"
	PENDING = "Pending"
	TEMPORARY_ERROR = "TemporaryError"
	MISSING_PREAUTH = "MissingPreAuth"
	MISSING_DELIVERY_REPORT = "MissingDeliveryReport"
	USER_IN_TRANSACTION = "UserInTransaction"
	CONNECTION_OFFLINE = "ConnectionOffline"
	INVALID_CREDENTIALS = "InvalidCredentials"
	INVALID_OTP = "InvalidOtp"
	CARD_PSP_ERROR = "CardPspError"
	MNO_ERROR = "MnoError"


from .model import Model

class StrexTransaction(Model):

    def _accepted_params(self):
        return [
            'transactionId',
            'sessionId',
            'correlationId',
            'shortNumber',
            'recipient',
            'content',
            'oneTimePassword',
            'deliveryMode',
            'statusCode',
            'detailedStatusCode',
            'smscTransactionId',
            'created',
            'lastModified',
            'merchantId',
            'serviceCode',
            'businessModel',
            'preAuthServiceId',
            'preAuthServiceDescription',
            'age',
            'isRestricted',
            'smsConfirmation',
            'invoiceText',
            'price',
            'timeout',
            'tags',
            'properties',
            'billed',
            'resultCode',
            'resultDescription',
        ]



from .helpers.http_client import HttpClient
from .models.lookup_result import LookupResult
from .models.keyword import Keyword
from .models.out_message import OutMessage
from .models.strex_merchant import StrexMerchant
from .models.in_message import InMessage
from .models.one_time_password import OneTimePassword
from .models.strex_transaction import StrexTransaction
from .models.public_key import PublicKey
from .models.oneclick_config import OneClickConfig
from .models.user_validity import UserValidity

name = "target365_sdk"

class ApiClient:
    PING = "api/ping"
    LOOKUP = "api/lookup"
    KEYWORDS = "api/keywords"
    OUT_MESSAGES = "api/out-messages"
    OUT_MESSAGE_EXPORT = "api/export/out-messages"
    IN_MESSAGES = "api/in-messages"
    PREPARE_MSISDNS = "api/prepare-msisdns"
    STREX_MERCHANTS = "api/strex/merchants"
    STREX_TRANSACTIONS = "api/strex/transactions"
    STREX_ONE_TIME_PASSWORDS = "api/strex/one-time-passwords"
    STREX_REGISTRATION_SMS = "api/strex/registrationsms"
    STREX_USER_INFO = "api/strex/validity"
    SERVER_PUBLIC_KEYS = "api/server/public-keys"
    CLIENT_PUBLIC_KEYS = "api/client/public-keys"
    ONECLICK_CONFIGS = "api/one-click/configs"

    NOT_FOUND = 404

    def __init__(self, base_uri, key_name, private_key):
        self.client = HttpClient(base_uri, key_name, private_key)

    def ping(self):
        """
        Pings the service and returns 'pong' string
        :return: string 'pong'.
        """

        response = self.client.get(self.PING)
        response.raise_for_status()

        return response.text  # returns the string "pong"

    def lookup(self, msisdn):
        """
        Looks up address info on a mobile phone number.
        :msisdn: Mobile phone number (required).
        :return: LookupResult object.
        """

        if msisdn is None:
            raise ValueError("msisdn")

        query = {"msisdn": msisdn}
        response = self.client.get_with_params(self.LOOKUP, query)

        if response.status_code == self.NOT_FOUND:
            return None

        response.raise_for_status()

        lookup_result = LookupResult(**response.json())
        return lookup_result

    def create_keyword(self, keyword):
        """
        Creates a new keyword.
        :keyword: Keyword object.
        :return: resulting keyword id string.
        """
        if keyword is None:
            raise ValueError("keyword")
        response = self.client.post(self.KEYWORDS, keyword)
        response.raise_for_status()

        return self._get_id_from_header(response.headers)

    def get_all_keywords(self, short_number_id=None, keyword=None, mode=None, tag=None):
        """
        Gets keywords.
        :short_number_id: Short number id to search on.
        :keyword: Keyword text to search for.
        :mode: Keyword mode to search for.
        :tag: Keyword tag to search for.
        :return: array of Keyword objects.
        """
        query = {}
        if short_number_id is not None:
            query["shortNumberId"] = short_number_id
        if keyword is not None:
            query["keywordText"] = keyword
        if mode is not None:
            query["mode"] = mode
        if tag is not None:
            query["tag"] = tag

        response = self.client.get_with_params(self.KEYWORDS, query)
        response.raise_for_status()
        return Keyword.from_list(response.json())

    def get_keyword(self, keyword_id):
        """
        Gets a keyword.
        :keyword_id: keyword id string.
        :return: Keyword object.
        """
        if keyword_id is None:
            raise ValueError("keywordId")

        response = self.client.get(self.KEYWORDS + "/" + keyword_id)
        if response.status_code == self.NOT_FOUND:
            return None

        response.raise_for_status()
        
        return Keyword(**response.json())

    def update_keyword(self, keyword):
        """
        Updates a keyword.
        :param keyword: Keyword object.
        """
        if keyword is None:
            raise ValueError("keyword")
        if keyword.keywordId is None:
            raise ValueError("keywordId")

        response = self.client.put(
            self.KEYWORDS + "/" + keyword.keywordId, keyword)

        response.raise_for_status()

    def delete_keyword(self, keyword_id):
        """
        Deletes a keyword.
        :keyword_id: Keyword id string.
        """
        if keyword_id is None:
            raise ValueError("keyword_id")

        response = self.client.delete(self.KEYWORDS + "/" + keyword_id)
        response.raise_for_status()

    def prepare_msisdns(self, msisdns):
        """
        Prepare MSISDNs for later sendings. This can greatly improve routing performance.
        :msisdns: MSISDNs to prepare as a string array.
        """
        if msisdns is None:
            raise ValueError("msisdns")

        response = self.client.post(self.PREPARE_MSISDNS, msisdns)
        response.raise_for_status()

    def create_out_message(self, out_message):
        """
        Creates a new out-message
        :out_message: OutMessage object.
        """
        if out_message is None:
            raise ValueError("out_message")

        response = self.client.post(self.OUT_MESSAGES, out_message)
        response.raise_for_status()

        return self._get_id_from_header(response.headers)

    def create_out_message_batch(self, out_messages):
        """
        Creates a new out-message batch.
        :out_messages: Array of OutMessage objects.
        """
        if out_messages is None:
            raise ValueError("out_messages")

        response = self.client.post(self.OUT_MESSAGES + "/batch", out_messages)
        response.raise_for_status()

    def get_out_message(self, transaction_id):
        """
        Gets an out-message by transaction id.
        :transaction_id: Transaction id string.
        :return: OutMessage object.
        """
        if transaction_id is None:
            raise ValueError("transactionId")

        response = self.client.get(self.OUT_MESSAGES + "/" + transaction_id)
        if response.status_code == self.NOT_FOUND:
            return None

        response.raise_for_status()

        return OutMessage(**response.json())

    def update_out_message(self, out_message):
        """
        Updates a future scheduled out-message.
        :out_message: OutMessage
        """
        if out_message is None:
            raise ValueError("out_message")
        if out_message.transactionId is None:
            raise ValueError("transactionId")

        response = self.client.put(self.OUT_MESSAGES + "/" + out_message.transactionId, out_message)
        response.raise_for_status()

    def delete_out_message(self, transaction_id):
        """
        Deletes a future sheduled out-message.
        :transaction_id: Out-message transaction id string.
        """
        if transaction_id is None:
            raise ValueError("transaction_id")

        response = self.client.delete(self.OUT_MESSAGES + "/" + transaction_id)
        response.raise_for_status()

    def get_out_message_export(self, from_date, to_date):
        """
        Gets out-message export in CSV format
        :from_date: From datetime in UTC
        :to_date: To datetime in UTC
        :return: string CSV data
        """
        query = {"from": from_date, "to": to_date}
        response = self.client.get_with_params(self.OUT_MESSAGE_EXPORT, query)
        response.raise_for_status()
        return response.text

    def get_in_message(self, short_number_id, transaction_id):
        """
        Gets an in-message.
        :short_number_id: string
        :transaction_id: string
        :return: InMessage object.
        """
        if short_number_id is None:
            raise ValueError("short_number_id")
        if transaction_id is None:
            raise ValueError("transaction_id")

        response = self.client.get(self.IN_MESSAGES + "/" + short_number_id + "/" + transaction_id)
        response.raise_for_status()
        return InMessage(**response.json())

    def get_strex_merchants(self):
        """
        Gets all merchant ids.
        :return: Array of StrexMerchant objects.
        """
        response = self.client.get(self.STREX_MERCHANTS)
        response.raise_for_status()
        return StrexMerchant.from_list(response.json())

    def get_strex_merchant(self, merchant_id):
        """
        Gets a strex merchant.
        :merchant_id: Merchant id string.
        :returns: StrexMerchant object.
        """
        if merchant_id is None:
            raise ValueError("merchant_id")

        response = self.client.get(self.STREX_MERCHANTS + "/" + merchant_id)

        if response.status_code == self.NOT_FOUND:
            return None

        response.raise_for_status()
        return StrexMerchant(**response.json())

    def save_strex_merchant(self, merchant):
        """
        Creates or updates a merchant.
        :merchant: StrexMerchant object.
        """
        if merchant is None:
            raise ValueError("merchant")
        if merchant.merchantId is None:
            raise ValueError("merchantId")

        response = self.client.put(self.STREX_MERCHANTS + "/" + merchant.merchantId, merchant)
        response.raise_for_status()

    def delete_strex_merchant(self, merchant_id):
        """
        DELETE /api/strex/merchants/{merchantId}
        Deletes a merchant
        :merchant_id: Merchant id string.
        """
        if merchant_id is None:
            raise ValueError("merchant_id")

        response = self.client.delete(self.STREX_MERCHANTS + "/" + merchant_id)
        response.raise_for_status()

    def create_one_time_password(self, one_time_password):
        """
        Creates a new one-time password.
        :one_time_password: OneTimePassword object.
        """

        if one_time_password is None:
            raise ValueError("one_time_password")
        if one_time_password.transactionId is None:
            raise ValueError("transactionId")
        if one_time_password.merchantId is None:
            raise ValueError("merchantId")
        if one_time_password.recipient is None:
            raise ValueError("recipient")
        if one_time_password.sender is None:
            raise ValueError("sender")
        if one_time_password.recurring is None:
            raise ValueError("recurring")

        response = self.client.post(self.STREX_ONE_TIME_PASSWORDS, one_time_password)
        response.raise_for_status()

    def get_one_time_password(self, transaction_id):
        """
        Gets a strex one-time password.
        :transaction_id: Transaction id string.
        :return: OneTimePassword object.
        """

        if transaction_id is None:
            raise ValueError("transaction_id")

        response = self.client.get(self.STREX_ONE_TIME_PASSWORDS + '/' + transaction_id)
        response.raise_for_status()
        return OneTimePassword(**response.json())

    def create_strex_transaction(self, transaction):
        """
        Creates a new strex transaction.
        :transaction: StrexTransaction object.
        :return: Transaction id string.
        """

        if transaction is None:
            raise ValueError("transaction")

        response = self.client.post(self.STREX_TRANSACTIONS, transaction)
        response.raise_for_status()
        return self._get_id_from_header(response.headers)

    def get_strex_transaction(self, transaction_id):
        """
        Gets a strex transaction.
        :transaction_id: Transaction id string.
        :return: StrexTransaction object.
        """

        if transaction_id is None:
            raise ValueError("transaction_id")

        response = self.client.get(self.STREX_TRANSACTIONS + '/' + transaction_id)
        response.raise_for_status()
        return StrexTransaction(**response.json())

    def reverse_strex_transaction(self, transaction_id):
        """
        Reverses a previous strex transaction.
        :transaction_id: Transaction id string.
        """

        if transaction_id is None:
            raise ValueError("transaction_id")

        response = self.client.delete(self.STREX_TRANSACTIONS + '/' + transaction_id)
        response.raise_for_status()

    def send_strex_registration_sms(self, strex_registration_sms):
        """
        Initiates Strex-registation by SMS.
        :strex_registration_sms: StrexRegistrationSms object.
        """

        if strex_registration_sms is None:
            raise ValueError("strex_registration_sms")

        response = self.client.post(self.STREX_REGISTRATION_SMS, strex_registration_sms)
        response.raise_for_status()

    def get_strex_user_info(self, merchant_id, recipient):
        """
        Gets Strex user validity.
        :merchant_id: Merchant id string.
        :recipient: MSISDN recipient string.
        :returns: UserValidity string.
        """

        if recipient is None:
            raise ValueError("recipient")

        query = {"recipient": recipient}

        if merchant_id is not None:
            query["merchantId"] = merchant_id

        response = self.client.get_with_params(self.STREX_USER_INFO, query)
        response.raise_for_status()
        return response.json();

    def get_server_public_key(self, key_name):
        """
        Gets a server public key.
        :key_name: public key name string.
        :return: PublicKey object.
        """

        if key_name is None:
            raise ValueError("key_name")

        response = self.client.get(self.SERVER_PUBLIC_KEYS + '/' + key_name)
        response.raise_for_status()
        return PublicKey(**response.json())

    def get_client_public_keys(self):
        """
        Gets all client public key.
        :return: Array of PublicKey objects.
        """
        response = self.client.get(self.CLIENT_PUBLIC_KEYS)
        response.raise_for_status()
        return PublicKey.from_list(response.json())

    def get_client_public_key(self, key_name):
        """
        Gets a client public key.
        :key_name: public key name string.
        :return: PublicKey object.
        """

        if key_name is None:
            raise ValueError("key_name")

        response = self.client.get(self.CLIENT_PUBLIC_KEYS + '/' + key_name)
        response.raise_for_status()
        return PublicKey(**response.json())

    def delete_client_public_key(self, key_name):
        """
        Deletes a client public key.
        :key_name: public key name string.
        :return:
        """

        if key_name is None:
            raise ValueError("key_name")

        response = self.client.delete(self.CLIENT_PUBLIC_KEYS + '/' + key_name)
        response.raise_for_status()

    def get_oneclick_config(self, config_id):
        """
        Gets a one-click config.
        :config_id: One-click config id string.
        :returns: OneClickConfig object.
        """

        if config_id is None:
            raise ValueError("config_id")

        response = self.client.get(self.ONECLICK_CONFIGS + "/" + config_id)

        if response.status_code == self.NOT_FOUND:
            return None

        response.raise_for_status()
        return OneClickConfig(**response.json())

    def save_oneclick_config(self, config):
        """
        Creates or updates a one-click config.
        :config: OneClickConfig object.
        """

        if config is None:
            raise ValueError("config")
        if config.configId is None:
            raise ValueError("configId")

        response = self.client.put(self.ONECLICK_CONFIGS + "/" + config.configId, config)
        response.raise_for_status()

    # noinspection PyMethodMayBeStatic,PyMethodMayBeStatic
    def _get_id_from_header(self, headers):
        """
        Returns the newly created resource's identifier from the Locaion header
        :returns: resource identifier
        """
        chunks = headers["Location"].split("/")
        return chunks[-1]



from .model import Model

class InMessage(Model):


    def _accepted_params(self):

        return [
            'messageId',
            'transactionId',
            'processed',
            'processAttempts',
            'sender',
            'recipient',
            'content',
            'keywordId',
            'isStopMessage',
            'created',
            'properties',
            'tags',
        ]



class UserValidity:
    # Not registered (0)
    Unregistered = "Unregistered"
    # Registered, but not valid for licensed purchase (1)
    Partial = "Partial"
    # Registered and valid for licensed purchase (2)
    Full = "Full"
    # Barred, not able to purchase (3)
    Barred = "Barred"



import pytest
import os
import uuid
from datetime import datetime
from datetime import timedelta
from ..api_client import ApiClient
from ..models.keyword import Keyword
from ..models.out_message import OutMessage
from ..models.out_message_strex import OutMessageStrex
from ..models.strex_merchant import StrexMerchant
from ..models.one_time_password import OneTimePassword
from ..models.strex_transaction import StrexTransaction
from ..models.status_codes import StatusCodes
from ..models.detailed_status_codes import DetailedStatusCodes
from ..models.oneclick_config import OneClickConfig
from ..models.strex_registration_sms import StrexRegistrationSms
from ..models.user_validity import UserValidity


@pytest.fixture
def valid_short_number_id():
    return 'NO-0000'


@pytest.fixture
def transaction_id():
    return '79f35793-6d70-423c-a7f7-ae9fb1024f3b'


@pytest.fixture(scope="session")
def random_transaction_id():
    return str(uuid.uuid4())


@pytest.fixture
def api_key_name():
    return os.environ['API_KEY_NAME']


@pytest.fixture
def api_private_key():
    return os.environ['API_PRIVATE_KEY']


@pytest.fixture
def client(api_key_name, api_private_key):
    base_uri = "https://test.target365.io/"

    client = ApiClient(base_uri, api_key_name, api_private_key)

    return client


def test_keyword_sequence(client, valid_short_number_id):
    keyword = Keyword()
    keyword.shortNumberId = valid_short_number_id
    keyword.keywordText = str(uuid.uuid4())
    keyword.mode = "Wildcard"
    keyword.forwardUrl = "https://tempuri.org"
    keyword.enabled = True
    keyword.created = "2018-04-12T12:00:00Z"
    keyword.lastModified = "2018-04-15T14:00:00Z"
    keyword.tags = ["Foo", "Bar"]

    # Create a keyword
    created_id = client.create_keyword(keyword)

    # Get the created keyword
    fetched_keyword = client.get_keyword(str(created_id))
    assert fetched_keyword.keywordText == keyword.keywordText

    # Update keyword
    fetched_keyword.keywordText = str(uuid.uuid4())
    client.update_keyword(fetched_keyword)
    updated_keyword = client.get_keyword(str(created_id))
    assert updated_keyword.keywordText == fetched_keyword.keywordText

    # Get all with filters returns record
    all_keywords = client.get_all_keywords(valid_short_number_id, None, "Wildcard", "Foo")
    assert len(all_keywords) > 0

    # Delete
    client.delete_keyword(str(created_id))

    # Trying to fetch returns None
    assert client.get_keyword(str(created_id)) is None


def test_out_message_sequence(client, valid_short_number_id):

    # Test alternative instantiation via constructor to ensure nested
    # models populate correctly
    out_message_alt_data = {
            'sender': '0000',
            'strex': {
                'merchantId': 'mer_test',
                'serviceCode': '14002'
            }
        }

    out_message_alt = OutMessage(
        **out_message_alt_data
    )

    assert type(out_message_alt.strex) == OutMessageStrex
    assert out_message_alt.strex.merchantId == 'mer_test'

    tomorrow = _add_days(datetime.utcnow(), 1)
    formatted = _format_datetime(tomorrow)

    # This is a nested model inside the OutMessage model
    out_message_strex = OutMessageStrex()
    out_message_strex.merchantId = 'mer_test'
    out_message_strex.serviceCode = '14002'
    out_message_strex.invoiceText = 'This is my invoice text'
    out_message_strex.price = 9.90
    out_message_strex.timeout = 10
    # out_message_strex.billed is read only

    # create
    out_message = OutMessage()
    out_message.strex = out_message_strex
    out_message.sender = "0000"
    out_message.recipient = "+4798079008"
    out_message.content = "Hi! This is a message from 0000 :)"
    out_message.sendTime = formatted
    identifier = client.create_out_message(out_message)

    # get
    fetched = client.get_out_message(identifier)

    assert type(fetched.strex) == OutMessageStrex

    fetched.content += fetched.content

    # update
    client.update_out_message(fetched)
    updated = client.get_out_message(identifier)
    assert updated.content == fetched.content

    # delete
    client.delete_out_message(identifier)
    assert client.get_out_message(identifier) is None

    # create batch
    t1 = uuid.uuid4()
    t2 = uuid.uuid4()
    t3 = uuid.uuid4()
    out_message1 = OutMessage()
    out_message1.transactionId = str(t1)
    out_message1.sender = "0000"
    out_message1.recipient = "+4798079008"
    out_message1.content = "Hi! This is a message from 0000 :)"
    out_message1.sendTime = formatted
    out_message2 = OutMessage()
    out_message2.transactionId = str(t2)
    out_message2.sender = "0000"
    out_message2.recipient = "+4798079008"
    out_message2.content = "Hi! This is a message from 0000 :)"
    out_message2.sendTime = formatted
    out_message3 = OutMessage()
    out_message3.transactionId = str(t3)
    out_message3.sender = "0000"
    out_message3.recipient = "+4798079008"
    out_message3.content = "Hi! This is a message from 0000 :)"
    out_message3.sendTime = formatted
    messages = [out_message1, out_message2, out_message3]
    client.create_out_message_batch(messages)

    client.delete_out_message(str(t1))
    client.delete_out_message(str(t2))
    client.delete_out_message(str(t3))


def test_out_message_export(client):
    from_date = "2020-08-22T00:00:00Z"
    to_date = "2020-08-23T00:00:00Z"
    csv = client.get_out_message_export(from_date, to_date)
    assert csv.startswith("SendTime,Sender,Recipient,RecipientPrefix,MessageParts,StatusCode,DetailedStatusCode,Operator,Tags") is True

def test_prepare_msisdns(client):
    client.prepare_msisdns(["+4798079008"])


def test_get_in_message(client, valid_short_number_id, transaction_id):
    in_message = client.get_in_message(valid_short_number_id, transaction_id)
    assert in_message.transactionId == transaction_id


def test_lookup_should_return_result(client):
    assert client.lookup("+4798079008") is not None


def test_create_one_time_password(client, random_transaction_id):
    one_time_password_data = {
        'transactionId': random_transaction_id,
        'merchantId': 'mer_test',
        'recipient': '+4798079008',
        'sender': 'Test',
        'recurring': False
    }

    one_time_password = OneTimePassword(**one_time_password_data)

    client.create_one_time_password(one_time_password)


def test_get_one_time_password(client, transaction_id):
    one_time_password = client.get_one_time_password(transaction_id)

    assert one_time_password.transactionId == transaction_id


def test_strex_transaction_sequence(client, random_transaction_id):
    strex_transaction_data = {
        "invoiceText": "Thank you for your donation",
        "merchantId": "mer_test",
        "price": 10,
        "timeout": 10,
        "recipient": "+4798079008",
        "serviceCode": "14002",
        "shortNumber": "0000",
        "transactionId": random_transaction_id,
        "oneTimePassword": "1234"
    }

    strex_transaction = StrexTransaction(**strex_transaction_data)

    client.create_strex_transaction(strex_transaction)

    strex_transaction = client.get_strex_transaction(random_transaction_id)
    assert strex_transaction.transactionId == random_transaction_id
    assert strex_transaction.timeout == 10

    # get_strex_transaction will wait up to 20 secs for trans to finish
    strex_transaction = client.get_strex_transaction(random_transaction_id)

    assert strex_transaction.detailedStatusCode == DetailedStatusCodes.DELIVERED

    # reverse_strex_transaction will wait up to 20 secs if trans isn't finished
    client.reverse_strex_transaction(random_transaction_id)

    # reversal transaction id is always original id prefixed by '-'
    reversal_transaction = client.get_strex_transaction("-" + random_transaction_id)
    
    assert reversal_transaction.statusCode == StatusCodes.REVERSED

def test_user_validity(client):
    validity = client.get_strex_user_info("mer_test", "+4799031520")
    assert validity == UserValidity.Full

def test_get_server_public_key(client):
    public_key = client.get_server_public_key('2021-07-27')

    assert public_key.accountId == 8


def test_get_client_public_keys(client, api_key_name):
    client_public_keys = client.get_client_public_keys()

    found_key = False
    for client_public_key in client_public_keys:
        if client_public_key.name == api_key_name:
            found_key = True

    assert found_key is True


def test_create_oneclick_config(client):
    config_data = {
				"configId": "APITEST",
				"shortNumber": "0000",
				"price": 99,
				"merchantId": "mer_test",
				"businessModel": "STREX-PAYMENT",
				"serviceCode": "14002",
				"invoiceText": "Donation test",
				"onlineText": "Buy directly",
				"offlineText": "Buy with PIN-code",
				"redirectUrl": "https://tempuri.org/php",
        "subscriptionPrice": 99,
        "subscriptionInterval": "monthly",
        "subscriptionStartSms": "Thanks for donating 99kr each month.",
				"recurring": False,
				"isRestricted": False,
				"age": 0,
    }

    config = OneClickConfig(**config_data)
    client.save_oneclick_config(config)


def test_get_oneclick_config(client):
    config = client.get_oneclick_config("APITEST")

    assert config.configId == "APITEST"


def test_get_one_time_password(client, transaction_id):
    one_time_password = client.get_one_time_password(transaction_id)

    assert one_time_password.transactionId == transaction_id


def test_send_strex_registration_sms(client, random_transaction_id):
    data = {
				"transactionId": random_transaction_id,
        "recipient": "+4798079008",
				"merchantId": "mer_test",
        "smsText": "Please register as a Strex-custom to continue.",
    }

    strex_registration_sms = StrexRegistrationSms(**data)
    client.send_strex_registration_sms(strex_registration_sms)
    

# Formats datetime object into utc string
# got from https://stackoverflow.com/q/19654578/1241791
def _format_datetime(d):
    return d.strftime('%Y-%m-%dT%H:%M:%S') + d.strftime('.%f')[:4] + 'Z'


def _add_days(d, days_count):
    return d + timedelta(days=days_count)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="target365_sdk",
    version="1.7.2",
    author="Target365",
    author_email="support@target365.no",
    description="Target365 SDK",
    long_description="Enables integration with Target365 online services.",
    long_description_content_type="text/markdown",
    url="https://github.com/Target365/sdk-for-python",
    packages=setuptools.find_packages(),
    install_requires=[
          'requests',
          'ecdsa',
          'jsonpickle',
      ],
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from abc import ABCMeta, abstractmethod

class Model:
    __metaclass__ = ABCMeta

    def __init__(self, **kwargs):

        args = self._init_preprocess(kwargs)

        if type(args) is not dict:
            raise Exception('_init_preprocess() must return a dict')

        for key, value in args.items():
            if key in self._accepted_params():
                setattr(self, key, value)

    def _init_preprocess(self, args):
        """
        sometimes additional work needs to be done when init an model such
        as instantiating nested models. You should override this method

        should return args (modify if needed)
        """
        return args

    @abstractmethod
    def _accepted_params(self):
        """
        Abstract method. This should return a list of all parameters which
        this model should accept
        """
        pass


    @classmethod
    def from_list(cls, items):
        """
        :param items:
        :type items: list
        :return: list
        """
        return [cls(**item) for item in items]



import requests
import ecdsa
import binascii
import time
import uuid
import base64
import hashlib
import jsonpickle

try:
    #python2
    from urllib import urlencode
except ImportError:
    #python3
    from urllib.parse import urlencode

class HttpClient:
    def __init__(self, base_uri, key_name, private_key):
        self.keyName = key_name
        self.privateKey = private_key
        self.base_uri = base_uri
        self.publicKey = ecdsa.SigningKey.from_string(
            binascii.unhexlify(self.privateKey), curve=ecdsa.NIST256p)

    def get(self, path):
        return requests.get(self._build_url(path), headers=self._get_auth_header("get", self._build_url(path)))

    def get_with_params(self, path, query_params):

        url = self._build_url(path)
        if len(query_params.keys()) > 0:
            url += "?"

        absolute_uri = (url + urlencode(query_params)).lower()
        return requests.get(
            self._build_url(path),
            params=query_params,
            headers=self._get_auth_header("get", absolute_uri)
        )

    def post(self, path, body):
        json_encoded = jsonpickle.encode(body, unpicklable=False)
        return requests.post(
            self._build_url(path),
            data=json_encoded,
            headers=self._get_auth_header("post", self._build_url(path), json_encoded)
        )

    def put(self, path, body):
        json_encoded = jsonpickle.encode(body,  unpicklable=False)
        return requests.put(
            self._build_url(path),
            data=json_encoded,
            headers=self._get_auth_header("put", self._build_url(path), json_encoded)
        )

    def delete(self, path):
        return requests.delete(
            self._build_url(path),
            headers=self._get_auth_header("delete", self._build_url(path))
        )

    def _build_url(self, path):
        return (self.base_uri + path)

    def _get_auth_header(self, method, uri, body=None):
        signature = self._get_signature(method, uri, body)
        return {"Authorization": "ECDSA " + signature}

    def _get_signature(self, method, uri, body=None):
        timestamp = int(time.time())
        nounce = uuid.uuid4()

        content_hash = ""
        if body is not None:
            content = body
            signature = hashlib.sha256(content.encode("utf-8")).digest()
            base64_encoded = base64.b64encode(signature)
            content_hash = base64_encoded.decode("utf-8")

        message = method + uri.lower() + str(timestamp) + str(nounce) + content_hash
        signature_string = base64.b64encode(self.publicKey.sign(
            message.encode("utf-8"), hashfunc=hashlib.sha256))
        the_signature = self.keyName + ":" + str(timestamp) + ":" + str(nounce) + ":" + signature_string.decode("utf-8")

        return the_signature
